# -*- coding: utf-8 -*-
# Generated by Django 1.9.1 on 2016-02-04 15:56
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djxami', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='message',
            options={'ordering': ['-created_at']},
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.1 on 2016-02-05 16:03
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djxami', '0003_message_stream'),
    ]

    operations = [
        migrations.AddField(
            model_name='message',
            name='expires',
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='message',
            name='permanent',
            field=models.BooleanField(default=False),
        ),
    ]



from django.shortcuts import render_to_response
from django.template import Library
from django.conf import settings
from django.utils.safestring import mark_safe

register = Library()


@register.inclusion_tag('djxami/init.html')
def djxami_init():
    return {
        'first_query_pause': getattr(settings, 'DJXAMI_FIRST_QUERY_PAUSE', 1000),
        'queries_interval': getattr(settings, 'DJXAMI_QUERIES_INTERVAL', 5000),
        'stream_interval': getattr(settings, 'DJXAMI_STREAM_INVETRAL', 1000),
    }



from djxami.models import Message
from jsonfield import JSONField
from django.db import models


class NottyMessage(Message):
    XAMI_TYPE = 'djnotty'
    scripts = models.CharField(max_length=500, blank=True)
    notty_options = JSONField(blank=True, null=True)

    def builders(self, builders):
        notty_opts = {}
        scripts = []
        for b in builders:
            if b.stream is True:
                self.stream = True

            notty_opts = b.make(notty_opts)
            if hasattr(b, 'script_name'):
                scripts.append(b.script_name)

        self.notty_options = notty_opts
        self.scripts = ','.join(scripts) if scripts else ''

    def apply_builder(self, builder, commit=True):
        self.notty_options.update(builder.make(self.notty_options))
        if commit:
            self.save()

    def as_json(self):
        return {'opts': self.notty_options, 'scripts': self.scripts.split(','), 'id': self.pk}

    def as_json_for_stream(self):
        return self.as_json()



# coding: utf-8
from __future__ import unicode_literals
from django.core.urlresolvers import reverse


class Builder(object):
    stream = False

    def make(self):
        raise NotImplementedError()


class Linked(Builder):
    script_name = 'djnotty_linked'

    def __init__(self, url):
        self.url = url

    def make(self, opts):
        opts.update({'linked': self.url})
        return opts


class Close(Builder):
    script_name = 'djnotty_close'

    def __init__(self, redirect=None):
        self.redirect = redirect

    def make(self, opts):
        if self.redirect:
            opts['redirect'] = self.redirect

        return opts


class Text(Builder):
    def __init__(self, text):
        self.text = text

    def make(self, opts):
        opts.update({'text': self.text})

        return opts


class ProgressBar(Builder):
    stream = True
    script_name = 'djnotty_progressbar'

    def __init__(self, text, current=0):
        self.current = current
        self.text = text

    def make(self, opts):
        opts.update({
            'current': self.current,
            'layout': 'bottomLeft',
            'type': 'alert',
            'template': '''
                <div class="noty_messsage">
                    <div class="progress">
                        <div class="progress-bar" role="progressbar" aria-valuenow="%s" aria-valuemin="0" aria-valuemax="100" style="width: %s%%;">
                            <span class="noty_text"></span>
                        </div>
                    </div>
                </div>
            ''' % (self.current, self.current),
            'text': ' %s%% - %s' % (self.current, self.text)

        })
        return opts



# -*- coding: utf-8 -*-
# Generated by Django 1.9.1 on 2016-02-04 14:32
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
from django.db.migrations.migration import SwappableTuple


def swappable_dependency_latest(value):
    """
    Turns a setting value into a dependency.
    """
    return SwappableTuple((value.split(".", 1)[0], "__latest__"), value)

class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('contenttypes', '__latest__'),
        ('auth', '__latest__'),
        swappable_dependency_latest(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Message',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('viewed', models.BooleanField(default=False)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('mark_as_viewed_at', models.DateTimeField(blank=True, null=True)),
                ('globally', models.BooleanField(default=False)),
                ('object_id', models.PositiveIntegerField(blank=True, null=True)),
                ('content_type', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType')),
            ],
        ),
        migrations.CreateModel(
            name='MessageToGroup',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('group', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='auth.Group')),
                ('message', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='djxami.Message')),
            ],
        ),
        migrations.CreateModel(
            name='MessageToUser',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('viewed', models.BooleanField(default=False)),
                ('message', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='message_to_user', to='djxami.Message')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='messages', to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.AddField(
            model_name='message',
            name='groups',
            field=models.ManyToManyField(through='djxami.MessageToGroup', to='auth.Group'),
        ),
        migrations.AddField(
            model_name='message',
            name='users',
            field=models.ManyToManyField(through='djxami.MessageToUser', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterIndexTogether(
            name='message',
            index_together=set([('content_type', 'object_id')]),
        ),
    ]



from datetime import timedelta
from django.contrib.auth.models import Group
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.conf import settings
from model_utils.managers import InheritanceManager
from django.utils.timezone import now
from django.conf import settings


class MessageObjects(InheritanceManager):
    def mark_as_viewed_for_object(self, instance):
        objects = Message.objects.filter(content_type=ContentType.objects.get_for_model(instance), object_id=instance.pk).all()
        for obj in objects:
            obj.viewed = True
            obj.save()

        return True


class Message(models.Model):
    XAMI_TYPE = 'original'

    viewed = models.BooleanField(default=False)
    groups = models.ManyToManyField(Group, through='MessageToGroup')
    users = models.ManyToManyField(settings.AUTH_USER_MODEL, through='MessageToUser')
    created_at = models.DateTimeField(auto_now_add=True, editable=False)
    mark_as_viewed_at = models.DateTimeField(blank=True, null=True, editable=True)
    globally = models.BooleanField(default=False)
    content_type = models.ForeignKey(ContentType, null=True, blank=True)
    stream = models.BooleanField(default=False)
    object_id = models.PositiveIntegerField(null=True, blank=True)
    expires = models.DateTimeField(null=True, blank=True)
    permanent = models.BooleanField(default=False)
    objects = MessageObjects()

    def for_users(self, users):
        if users:
            for u in users:
                MessageToUser.objects.create(user=u, message=self)

    def for_groups(self, groups):
        if groups:
            for g in groups:
                MessageToGroup.objects.create(group=g, message=self)

    def for_object(self, model):
        cnt = ContentType.objects.get_for_model(model)
        self.content_type = cnt
        self.object_id = model.pk

    def set_lifetime(self, timedelta_object):
        self.expires = now() + timedelta_object

    def as_json(self):
        return {'text': 'msg', 'stream': False}

    def as_json_for_stream(self):
        return {}

    def save(self, *args, **kwargs):
        if not self.expires and not self.permanent:
            self.set_lifetime(timedelta(days=getattr(settings, 'DJXAMI_MESSAGE_LIFETIME_DAYS', 30)))
        return super(Message, self).save(*args, **kwargs)

    class Meta:
        index_together = (('content_type', 'object_id'),)
        ordering = ['-created_at']
        # abstract = True


class MessageToUser(models.Model):
    message = models.ForeignKey(Message, related_name='message_to_user')
    user = models.ForeignKey(settings.AUTH_USER_MODEL, related_name='messages')
    viewed = models.BooleanField(default=False)


class MessageToGroup(models.Model):
    message = models.ForeignKey(Message)
    group = models.ForeignKey(Group)



# -*- coding: utf-8 -*-
# Generated by Django 1.9.1 on 2016-02-04 17:38
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djxami', '0002_auto_20160204_1556'),
    ]

    operations = [
        migrations.AddField(
            model_name='message',
            name='stream',
            field=models.BooleanField(default=False),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.1 on 2016-02-04 14:43
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import jsonfield.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('djxami', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='NottyMessage',
            fields=[
                ('message_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='djxami.Message')),
                ('scripts', models.CharField(blank=True, max_length=500)),
                ('notty_options', jsonfield.fields.JSONField(blank=True, null=True)),
            ],
            bases=('djxami.message',),
        ),
    ]



import os
from setuptools import setup

os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='djxami',
    version='0.0.5',
    packages=['djxami','djnotty'],
    url='https://github.com/xacce/djxami',
    include_package_data=True,
    license='MIT',
    author='xacce',
    description='Create popup notifications.',
    long_description_markdown_filename='README.md',
    install_requires=['django-jsonfield']
)



from django.conf.urls import url, patterns
from djxami import views

urlpatterns = patterns('',
                       url('^$', views.Messages.as_view(), name="messages"),
                       url('^close/$', views.Close.as_view(), name="close"),
                       url('^stream/$', views.Stream.as_view(), name="stream"),
                       )



# coding: utf-8
from __future__ import unicode_literals
from operator import or_
from django.http.response import HttpResponse
from django.shortcuts import get_object_or_404
from django.views import generic
from djxami import models
from django.db.models import Q
from django.conf import settings
from django.utils.timezone import now
from datetime import timedelta
import json


class Stream(generic.View):
    def post(self, request, *args, **kwargs):
        message = models.Message.objects.get_subclass(pk=self.request.POST.get('id'))
        data = {'type': message.XAMI_TYPE, 'id': message.pk}
        data.update(message.as_json_for_stream())
        return HttpResponse(json.dumps(data), content_type='application/json')


class Messages(generic.View):
    def get_queryset(self):
        qs = models.Message.objects.filter(viewed=False)

        flt = [Q(globally=True)]
        if self.request.user.is_authenticated():
            qs = qs.filter(~Q(message_to_user__in=models.MessageToUser.objects.filter(user=self.request.user, viewed=True)))
            flt.append(Q(groups__in=self.request.user.groups.all()))
            flt.append(Q(users__in=[self.request.user]))
        return qs.filter(reduce(or_, flt)).select_subclasses()

    def get(self, *args, **kwargs):
        messages = []
        for message in self.get_queryset()[:10]:
            data = {'type': message.XAMI_TYPE, 'id': message.pk, 'stream': message.stream}
            data.update(message.as_json())
            messages.append(data)

        return HttpResponse(json.dumps(messages), content_type='application/json')


class Close(generic.View):
    model = models.Message

    def post(self, *args, **kwargs):
        response = HttpResponse(json.dumps({'status': 'success'}), content_type='application/json')
        message = get_object_or_404(models.Message, pk=int(self.request.POST.get('id')))
        if self.request.user.is_authenticated():
            try:
                rel = models.MessageToUser.objects.get(user=self.request.user, message=message)
            except models.MessageToUser.DoesNotExist:
                rel = models.MessageToUser(user=self.request.user, message=message)

            rel.viewed = True
            rel.save()
        else:
            viewed = [unicode(message.pk)]
            in_cookies = self.request.COOKIES.get('djxami', None)
            if in_cookies and len(in_cookies) < 1000:
                viewed += in_cookies.split(':')
            print viewed
            response.set_cookie('djxami',
                                str(':'.join(set(viewed))),
                                expires=now() + timedelta(days=365),
                                domain=settings.SESSION_COOKIE_DOMAIN,
                                secure=settings.SESSION_COOKIE_SECURE or None)

        return response
